import * as d3 from 'd3';

const svg = d3.select('svg');

const colorScale = d3.scaleOrdinal()
	.domain(['ROOT', 'STP', 'Expert'])
	.range(['red', 'maroon', 'steelblue']);

const width = +svg.attr('width');
const height = +svg.attr('height');

var graph = true;

var data = {
      "id": 559,
      "rid": "#29:42",
      "name": "Blockchain ",
      "type": "STP",
      "children": [
        {
          "id": 1538,
          "rid": "#33:106",
          "name": "Daniel Weber",
          "type": "Expert",
          "children": [
            {
              "id": 559,
              "rid": "#29:42",
              "name": "Blockchain ",
              "type": "STP"
            },
            {
              "id": 609,
              "rid": "#31:48",
              "name": "Mobile Apps",
              "type": "STP"
            }
          ]
        },
        {
          "id": 1539,
          "rid": "#34:106",
          "name": "Tim Jester-Pfadt",
          "type": "Expert",
          "children": [
            {
              "id": 559,
              "rid": "#29:42",
              "name": "Blockchain ",
              "type": "STP"
            }
          ]
        },
        {
          "id": 1540,
          "rid": "#35:106",
          "name": "NATALI MELGAREJO DIAZ",
          "type": "Expert",
          "children": [
            {
              "id": 559,
              "rid": "#29:42",
              "name": "Blockchain ",
              "type": "STP"
            }
          ]
        },
        {
          "id": 1223,
          "rid": "#38:66",
          "name": "Bjoern Ebeling",
          "type": "Expert",
          "children": [
            {
              "id": 337,
              "rid": "#27:18",
              "name": "AI",
              "type": "STP"
            },
            {
              "id": 404,
              "rid": "#30:26",
              "name": "Design Thinking",
              "type": "STP"
            },
            {
              "id": 559,
              "rid": "#29:42",
              "name": "Blockchain ",
              "type": "STP"
            },
            {
              "id": 575,
              "rid": "#29:44",
              "name": "Merck STP",
              "type": "STP"
            },
            {
              "id": 610,
              "rid": "#32:48",
              "name": "RPA",
              "type": "STP"
            },
            {
              "id": 620,
              "rid": "#26:50",
              "name": "IoT",
              "type": "STP"
            },
            {
              "id": 661,
              "rid": "#27:55",
              "name": "Text Mining",
              "type": "STP"
            }
          ]
        },
        {
          "id": 1331,
          "rid": "#34:80",
          "name": "Oliver Hoerr",
          "type": "Expert",
          "children": [
            {
              "id": 404,
              "rid": "#30:26",
              "name": "Design Thinking",
              "type": "STP"
            },
            {
              "id": 559,
              "rid": "#29:42",
              "name": "Blockchain ",
              "type": "STP"
            },
            {
              "id": 575,
              "rid": "#29:44",
              "name": "Merck STP",
              "type": "STP"
            },
            {
              "id": 699,
              "rid": "#25:60",
              "name": "Chatbot",
              "type": "STP"
            }
          ]
        }
      ]
    };

var nodes = d3.range(5).map(() => (
  {
    id: Math.round(Math.random()*100),
    radius: Math.round(Math.random()*20 + 10),
  })
);

var root = d3.hierarchy(data)

nodes = root.descendants();
var links = root.links();

const g = svg.append('g');

svg.call(d3.zoom().on('zoom', () => {
  console.log('zoomed')
	g.attr('transform', d3.event.transform)
}));

function renderLines(){
  console.log({links})
	var lines = g
    .selectAll('line')
    .data(links)

  lines.enter()
    .append('line')
    .attr('stroke', 'black')
    .merge(lines)
    .attr('x1', function(d) {return d.source.x;})
  	.attr('y1', function(d) {return d.source.y;})
  	.attr('x2', function(d) {return d.target.x;})
  	.attr('y2', function(d) {return d.target.y;});

  lines.exit().remove()
}

function renderNodes(){
	var circles = g
    .selectAll('circle')
    .data(nodes)

  circles.enter()
    .append('circle')
    .attr('r', d => d.depth === 0 ? 20 : 10)
  	.call(d3.drag()
          .on("start", dragStarted)
          .on('drag', dragging)
          .on('end', dragEnded)
    )
    .merge(circles)
    .attr('fill', d => colorScale(d.depth === 0 ? 'ROOT' : d.data.type))
    .attr('cx', d => d.x)
    .attr('cy', d => d.y);

  circles.exit().remove();
}

function ticked() {
  renderLines();
  renderNodes();
}

var dragStarted = (d) => {
    	if (!d3.event.active) simulation.alphaTarget(0.3).restart();
			d.fx = d.x;
      d.fy = d.y
		},
    dragging = (d) => {
    	d.fx = d3.event.x;
      d.fy = d3.event.y
    },
		dragEnded = (d) => {
    	if (!d3.event.active){
      	simulation.alphaTarget(0);
        simulation.restart();
      }
      d.fx = null;
    	d.fy = null
    }

var simulation = d3.forceSimulation(nodes)
  .force('charge', d3.forceManyBody().strength(-50))
	//.force('charge', d3.forceManyBody().strength())
  .force('center', d3.forceCenter(width / 2, height / 2))
	.force('link', d3.forceLink().links(links))
	.force('collision', d3.forceCollide().radius(function(d) {
    return d.depth === 0 ? 40 : 20
  }))
  .on('tick', () => ticked());

function changeView(){
  graph = !graph;
  links = graph ? root.links() : [];
  if(!graph){
    simulation.force('charge', d3.forceManyBody().strength(250))
		.force('link', () => {})
		.force('collision', d3.forceCollide().radius(function(d) {
    	return d.depth === 0 ? 20 : 10
  	}))
  }else{
    simulation.force('charge', d3.forceManyBody().strength(-50))
			.force('center', d3.forceCenter(width / 2, height / 2))
			.force('link', d3.forceLink().links(links))
			.force('collision', d3.forceCollide().radius(function(d) {
    		return d.depth === 0 ? 40 : 20
  		}))
  }
  //simulation.stop();
  //simulation.restart().on('tick', () => ticked());
  ticked();
  simulation.tick(1);
};

window.addEventListener('switch', changeView, false);





